/*****************************************************************************
 *
 * Author        : Copyright (c) Olli Parviainen
 * Author e-mail : oparviai @ iki.fi
 * File created  : 07-Jan-2002
 *
 * $Id: MessageCtrlDlg.cpp 326 2015-10-18 15:38:27Z olli $
 *
 * License:
 *
 * MessageCtrl example tool for controlling PaceMaker plug-in with Windows 
 * messages.
 * Copyright (c) Olli Parviainen
 *
 * This source code can be freely used, modified and further distributed.
 *
 * WWW: http://www.iki.fi/oparviai/pacemaker
 *
 *****************************************************************************/

#include "stdafx.h"

#include <math.h>

#include "MessageCtrl.h"
#include "MessageCtrlDlg.h"
#include "pm_msgapi.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define TWOPI 6.283185307179586476925286766559

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
    CAboutDlg();

// Dialog Data
    //{{AFX_DATA(CAboutDlg)
    enum { IDD = IDD_ABOUTBOX };
    //}}AFX_DATA

    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CAboutDlg)
    protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    //}}AFX_VIRTUAL

// Implementation
protected:
    //{{AFX_MSG(CAboutDlg)
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
    //{{AFX_DATA_INIT(CAboutDlg)
    //}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CAboutDlg)
    //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
    //{{AFX_MSG_MAP(CAboutDlg)
        // No message handlers
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMessageCtrlDlg dialog

CMessageCtrlDlg::CMessageCtrlDlg(CWnd* pParent /*=NULL*/)
    : CDialog(CMessageCtrlDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CMessageCtrlDlg)
        // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
    hPaceMaker = 0;
    phase = 0;
}

void CMessageCtrlDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CMessageCtrlDlg)
    DDX_Control(pDX, IDC_BUTTON1, m_button1);
    //}}AFX_DATA_MAP
    DDX_Control(pDX, IDC_STATIC_INFO, m_StaticInfo);
}

BEGIN_MESSAGE_MAP(CMessageCtrlDlg, CDialog)
    //{{AFX_MSG_MAP(CMessageCtrlDlg)
    ON_WM_SYSCOMMAND()
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    ON_BN_CLICKED(IDC_BUTTON1, OnButton1)
    ON_WM_TIMER()
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMessageCtrlDlg message handlers

BOOL CMessageCtrlDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Add "About..." menu item to system menu.

    // IDM_ABOUTBOX must be in the system command range.
    ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
    ASSERT(IDM_ABOUTBOX < 0xF000);

    CMenu* pSysMenu = GetSystemMenu(FALSE);
    if (pSysMenu != NULL)
    {
        CString strAboutMenu;
        strAboutMenu.LoadString(IDS_ABOUTBOX);
        if (!strAboutMenu.IsEmpty())
        {
            pSysMenu->AppendMenu(MF_SEPARATOR);
            pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
        }
    }

    // Set the icon for this dialog.  The framework does this automatically
    //  when the application's main window is not a dialog
    SetIcon(m_hIcon, TRUE);         // Set big icon
    SetIcon(m_hIcon, FALSE);        // Set small icon
    
    // TODO: Add extra initialization here
    
    return TRUE;  // return TRUE  unless you set the focus to a control
}

void CMessageCtrlDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
    if ((nID & 0xFFF0) == IDM_ABOUTBOX)
    {
        CAboutDlg dlgAbout;
        dlgAbout.DoModal();
    }
    else
    {
        CDialog::OnSysCommand(nID, lParam);
    }
}



// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CMessageCtrlDlg::OnPaint() 
{
    if (IsIconic())
    {
        CPaintDC dc(this); // device context for painting

        SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

        // Center icon in client rectangle
        int cxIcon = GetSystemMetrics(SM_CXICON);
        int cyIcon = GetSystemMetrics(SM_CYICON);
        CRect rect;
        GetClientRect(&rect);
        int x = (rect.Width() - cxIcon + 1) / 2;
        int y = (rect.Height() - cyIcon + 1) / 2;

        // Draw the icon
        dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
        CDialog::OnPaint();
    }
}



// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CMessageCtrlDlg::OnQueryDragIcon()
{
    return (HCURSOR) m_hIcon;
}



// Starts manic pacemaker tweaking
void CMessageCtrlDlg::start()
{
    int num;
    HWND *hs = seekPaceMakerWindows(num);

    if (num == 0) 
    {
        MessageBox("PaceMaker plug-in not active or incompatible plug-in version", "Error", MB_ICONERROR);
        return;
    }
    hPaceMaker = hs[0];  // let's now take the 1st found PaceMaker (in case several PaceMakers were found)

    // get handle of master Winamp/player window
    HWND hWinamp = (HWND)::SendMessage(hPaceMaker, PM_GET_PARENT_WINDOW, 0, (LPARAM)&hWinamp);

    if (hWinamp)
    {
        char buffer[64];
        char buffer2[64];
        char buffer3[128];
        ::GetWindowText(hWinamp, buffer, 64);
        ::GetClassName(hWinamp, buffer2, 64);

        wsprintf(buffer3, "Player title: %s\r\nClass info: %s", buffer, buffer2);

        m_StaticInfo.SetWindowText(buffer3);
    }



    // Set the PaceMaker slider precisions to that of 0.01 for more accurate
    // control

   ::PostMessage(hPaceMaker, PM_SET_PRECISION, 0, 3);   // sets tempo slider precision
   ::PostMessage(hPaceMaker, PM_SET_PRECISION, 1, 3);   // sets pitch slider precision
   ::PostMessage(hPaceMaker, PM_SET_PRECISION, 2, 3);   // sets speed slider precision

    // Start timer
    timerID = SetTimer(1, 50, NULL);
    m_button1.SetWindowText("&Stop");
}


// Stops pacemaker tweaking
void CMessageCtrlDlg::stop()
{
    KillTimer(timerID);
    // reset PaceMaker controls to their middle positions
    ::PostMessage(hPaceMaker, PM_RESET, 0, 0);
    hPaceMaker = NULL;
    m_button1.SetWindowText("&Start");
}


void CMessageCtrlDlg::OnButton1() 
{
    if (hPaceMaker == 0) 
    {
        start();
    } 
    else 
    {
        stop();
    }
}


void CMessageCtrlDlg::OnTimer(UINT nIDEvent) 
{
    float tempo, pitch, speed;

    phase += 0.15f;

    tempo = 25.0f * (float)sin(phase);                      // wiggle the tempo between -25%..+25% 
    pitch = 6.0f * (float)sin(phase + TWOPI / 3.0);         // wiggle the pitch between -6..6 semitones 
    speed = 25.0f * (float)sin(phase + 2.0 * TWOPI / 3.0);  // wiggle the speed between -25%..+25% 

    ::PostMessage(hPaceMaker, PM_SET_TEMPO, 0, (int)(tempo * SCALE));
    ::PostMessage(hPaceMaker, PM_SET_PITCH, 0, (int)(pitch * SCALE));
    ::PostMessage(hPaceMaker, PM_SET_SPEED, 0, (int)(speed * SCALE));
    
//  CDialog::OnTimer(nIDEvent);
}
