/*****************************************************************************
 *
 * Author        : Copyright (c) Olli Parviainen
 * Author e-mail : oparviai @ iki.fi
 * File created  : 07-Jan-2002
 *
 * $Id: pm_msgapi.cpp 322 2015-10-16 16:17:04Z olli $
 *
 * License:
 *
 * MessageCtrl example tool for controlling PaceMaker plug-in with Windows 
 * messages.
 * Copyright (c) Olli Parviainen
 *
 * This source code can be freely used, modified and further distributed.
 *
 * WWW: http://www.iki.fi/oparviai/pacemaker
 *
 *****************************************************************************/

#include "pm_msgapi.h"

// test if the window is really the PaceMaker window by sending a identification message
static BOOL testIfPaceMaker(HWND hwnd)
{
    DWORD response;
    int result;

    result = SendMessageTimeout(hwnd, PM_IDENTIFY, 0, 0, SMTO_ABORTIFHUNG, 1000, &response);
    if (result == 0) return FALSE;  // no answer
    if (response == 0x1770) return TRUE;    // expected result => it's pacemaker
    return FALSE;   // wrong answer
}



typedef struct
{
    int num;
    HWND h[32];
} SEEKSTRUCT;


// callback routine for going through the active desktop windows
static BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
    char title[32];

    ::GetWindowText(hwnd, title, 31);
    if (_strnicmp(title, "Pacemaker", 9) == 0) 
    {
        if (testIfPaceMaker(hwnd)) 
        {
            SEEKSTRUCT *pss = (SEEKSTRUCT *)lParam;
            pss->h[pss->num] = hwnd;
            pss->num ++;
            if (pss->num >= 32) return FALSE;   // enough PaceMakers already
        }
    }
    return TRUE;
}


// Returns an array of handles to all open PaceMaker plug-in windows, or NULL if none found
HWND *seekPaceMakerWindows(int &numOfFoundPaceMakers)
{
    static SEEKSTRUCT ss = {0, NULL};
    ss.num = 0;
    ss.h[0] = NULL;

    ::EnumWindows(EnumWindowsProc, (long)&ss);

    numOfFoundPaceMakers = ss.num;

    return (ss.num == 0) ? NULL : ss.h;
}

